// Copyright (c) 2011 Firebase.co and Contributors - http://www.firebase.co
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

var TestExpression = require('./TestExpression.js')
var AbortScenarioError = require('./AbortScenarioError.js')
var AssertError = require('fire-assert').AssertError
/*
 * Base type for When, And and Then step expressions. All derived types must implement createStepContext(currentContext, readyCallback(stepContext))
 */

function TestStepExpression() {
	
}

TestStepExpression.prototype = new TestExpression()
TestStepExpression.prototype.onPrepareInput = function() {
	var self = this
	this.inputExpression.scopeBypass = true
	var stepContext = this.stepContext
	this.setCurrentResult(stepContext.topic)
	this.inputExpression.errorCallback = function(err) {
		if(err.error instanceof AssertError) {
			stepContext.error = err.error
			self.raiseError(new AbortScenarioError())
		} else if(err.error instanceof AbortScenarioError) {
			stepContext.error = true
			// let the error bubble up to another Step or Scenario
			self.bubbleUpError(err)
		} else {
			// let the error bubble up...
			self.bubbleUpError(err)
		}
	}
}
TestStepExpression.prototype.execute = function() {
	var testContext = this.getTestContext()
	var self = this
	this.createStepContext(testContext, function(stepContext) {
		self.setTestContext(stepContext)
		self.stepContext = stepContext
		self.runInput(function() {
				self.bypass()
			})
	})
	
}

module.exports = TestStepExpression